@@ -10,13 +10,14 @@ import multiprocessing as mp
 from pyrasta.crs import proj4_from
 from pyrasta.io_.files import _copy_to_file
 from pyrasta.tools.calculator import _op, _raster_calculation
-from pyrasta.tools.clip import _clip_raster_by_extent
+from pyrasta.tools.clip import _clip_raster_by_extent, _clip_raster_by_mask
 from pyrasta.tools.conversion import _resample_raster, _padding, _rescale_raster, \
     _align_raster, _extract_bands, _merge_bands, _read_array, _xy_to_2d_index, _read_value_at, \
     _project_raster
-from pyrasta.tools.exceptions import RasterBaseError
+from pyrasta.exceptions import RasterBaseError
 from pyrasta.tools.merge import _merge
-from pyrasta.tools.stats import _histogram
+from pyrasta.tools.rasterize import _rasterize
+from pyrasta.tools.stats import _histogram, _zonal_stats
 from pyrasta.tools.windows import _windowing
 from pyrasta.utils import lazyproperty, grid
 
@@ -95,26 +96,31 @@ class RasterBase:
 
         return _align_raster(self, other)
 
-    def clip(self, bounds=None, mask=None, by_extent=True):
+    def clip(self, bounds=None, mask=None, no_data=-999, all_touched=True):
         """ Clip raster
 
         Parameters
         ----------
         bounds: tuple
             tuple (x_min, y_min, x_max, y_max) in map units
-        mask:
+        mask: geopandas.GeoDataFrame
             Valid mask layer
-        by_extent: bool
-            if True, clip raster by extent, otherwise use mask layer
+        no_data: int or float
+            No data value
+        all_touched: bool
+            if True, all touched pixels within layer boundaries are burnt,
+            when clipping raster by mask
 
         Returns
         -------
 
         """
-        if by_extent:
-            return _clip_raster_by_extent(self, bounds)
+        if bounds is not None:
+            return _clip_raster_by_extent(self, bounds, no_data)
+        elif mask is not None:
+            return _clip_raster_by_mask(self, mask, no_data, all_touched)
         else:
-            pass  # TODO: clip raster by mask layer
+            raise ValueError("Either bounds or mask must be set")
 
     def extract_bands(self, bands):
         """ Extract bands as multiple rasters
@@ -219,11 +225,53 @@ class RasterBase:
         """
         return _padding(self, pad_x, pad_y, value)
 
+    @classmethod
+    def rasterize(cls, layer, projection, x_size, y_size, geo_transform,
+                  burn_values=None, attribute=None,
+                  gdal_driver=gdal.GetDriverByName("Gtiff"), nb_band=1,
+                  data_type=gdal.GetDataTypeByName("Float32"), no_data=-999,
+                  all_touched=True):
+        """ Rasterize geographic layer
+
+        Parameters
+        ----------
+        layer: geopandas.GeoDataFrame or gistools.layer.GeoLayer
+            Geographic layer to be rasterized
+        projection: str
+            Projection as a WKT string
+        x_size: int
+            Raster width
+        y_size: int
+            Raster height
+        geo_transform: tuple
+        burn_values: list[float] or list[int], default None
+            List of values to be burnt in each band, excusive with attribute
+        attribute: str, default None
+            Layer's attribute to be used for values to be burnt in raster,
+            excusive with burn_values
+        gdal_driver: osgeo.gdal.Driver, default GeoTiff
+            GDAL driver
+        nb_band: int, default 1
+            Number of bands
+        data_type: int, default "Float32"
+            GDAL data type
+        no_data: int or float, default -999
+            No data value
+        all_touched: bool
+
+        Returns
+        -------
+
+        """
+        return _rasterize(cls, layer, burn_values, attribute, gdal_driver, projection,
+                          x_size, y_size, nb_band, geo_transform, data_type, no_data,
+                          all_touched)
+
     @classmethod
     def raster_calculation(cls, rasters, fhandle, window_size=1000,
                            gdal_driver=gdal.GetDriverByName("Gtiff"),
                            data_type=gdal.GetDataTypeByName('Float32'),
-                           no_data=-999, **kwargs):
+                           no_data=-999, showprogressbar=True, **kwargs):
         """ Raster expression calculation
 
         Description
@@ -246,6 +294,8 @@ class RasterBase:
             GDAL data type for output raster
         no_data: int or float
             no data value in resulting raster
+        showprogressbar: bool
+            if True, show progress bar
         kwargs:
             fhandle keyword arguments (if any)
 
@@ -255,22 +305,25 @@ class RasterBase:
             New temporary instance
         """
         return _raster_calculation(cls, rasters, fhandle, window_size,
-                                   gdal_driver, data_type, no_data, **kwargs)
+                                   gdal_driver, data_type, no_data, showprogressbar, **kwargs)
 
-    def read_array(self, bounds=None):
+    def read_array(self, band=None, bounds=None):
         """ Write raster to numpy array
 
         Parameters
         ----------
+        band: int
+            Band number. If None, read all bands into multidimensional array.
         bounds: tuple
-            tuple as (x_min, y_min, x_max, y_max) in map units
+            tuple as (x_min, y_min, x_max, y_max) in map units. If None, read
+            the whole raster into array
 
         Returns
         -------
         numpy.ndarray
 
         """
-        return _read_array(self, bounds)
+        return _read_array(self, band, bounds)
 
     def read_value_at(self, x, y):
         """ Read value in raster at x/y map coordinates
@@ -416,6 +469,47 @@ class RasterBase:
         """
         return _xy_to_2d_index(self, x, y)
 
+    def zonal_stats(self, layer, band=1, stats=None, customized_stats=None,
+                    no_data=-999, all_touched=True, show_progressbar=True,
+                    nb_processes=mp.cpu_count()):
+        """ Compute zonal statistics
+
+        Compute statistic among raster values
+        within each feature of given geographic layer
+
+        Parameters
+        ----------
+        layer: geopandas.GeoDataFrame or gistools.layer.GeoLayer
+            Geographic layer
+        band: int
+            Band number
+        stats: list[str]
+            list of valid statistic names
+            "mean", "median", "min", "max", "sum", "std"
+        customized_stats: dict
+            User's own customized statistic functions
+            as {'your_function_name': function}
+        no_data: int or float
+            No data value
+        all_touched: bool
+            Whether to include every raster cell touched by a geometry, or only
+            those having a center point within the polygon.
+        show_progressbar: bool
+            If True, show progress bar status
+        nb_processes: int
+            number of processes for multiprocessing
+
+        Returns
+        -------
+        dict[list]
+            Dictionary with each statistic as a list corresponding
+            to the values for each feature in layer
+
+        """
+        if stats is not None:
+            return _zonal_stats(self, layer, band, stats, customized_stats,
+                                no_data, all_touched, show_progressbar, nb_processes)
+
     @property
     def crs(self):
         """ Return Coordinate Reference System
@@ -501,6 +595,13 @@ class RasterBase:
         return [self._gdal_dataset.GetRasterBand(band + 1).ComputeStatistics(False)[3]
                 for band in range(self.nb_band)]
 
+    @lazyproperty
+    def projection(self):
+        """ Get projection as a WKT string
+
+        """
+        return self._gdal_dataset.GetProjection()
+
     @lazyproperty
     def x_origin(self):
         return self.geo_transform[0]
