@@ -115,7 +115,7 @@ def _project_raster(raster, out_file, new_crs):
     gdal.Warp(out_file, raster._gdal_dataset, dstSRS=srs_from(new_crs))
 
 
-def _read_array(raster, bounds):
+def _read_array(raster, band, bounds):
     """ Read array from raster
 
     """
@@ -127,13 +127,19 @@ def _read_array(raster, bounds):
         reverse_transform = ~forward_transform
         px_min, py_max = reverse_transform * (x_min, y_min)
         px_max, py_min = reverse_transform * (x_max, y_max)
-        x_size = int(px_max) - int(px_min)
-        y_size = int(py_max) - int(py_min)
+        x_size = int(px_max - px_min) + 1
+        y_size = int(py_max - py_min) + 1
 
-        return raster._gdal_dataset.ReadAsArray(px_min,
-                                                py_min,
-                                                x_size,
-                                                y_size)
+        if band is not None:
+            return raster._gdal_dataset.GetRasterBand(band).ReadAsArray(int(px_min),
+                                                                        int(py_min),
+                                                                        x_size,
+                                                                        y_size)
+        else:
+            return raster._gdal_dataset.ReadAsArray(int(px_min),
+                                                    int(py_min),
+                                                    x_size,
+                                                    y_size)
 
 
 def _read_value_at(raster, x, y):
